"""Standalone script to produce Gantt Charts from AutoSubmit logs.

Example usage:

    $ python create_ganttchart.py o005 --chunks 38 52 --ignore AQUA

This produces the figure posted in:

    https://earth.bsc.es/gitlab/digital-twins/de_340-2/simulation-runs/-/issues/82#note_368241

Full options for plot creation:

    $ python create_ganttchart.py -h
"""

from argparse import ArgumentParser, RawDescriptionHelpFormatter
from pathlib import Path

import matplotlib.pyplot as plt

from as_analytics.logreport import get_aslogs_df, jobs_df_from_aslogs_df
from as_analytics.plots import plot_job_gantt_chart
from as_analytics.utils import get_available_expids
from as_analytics.paths import LOG_DIR

# Prefixes of files to ignore in Gantt Chart by default
IGNORE_PREFIX = (".", "ls", "config", "githook", "mother_request", "only_lra")


def _get_parser() -> ArgumentParser:
    parser = ArgumentParser(
        description="Gantt Chart plotter for Autosubmit logs.",
        formatter_class=RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "expid",
        type=str,
        choices=get_available_expids(),
        help="Experiment id",
    )
    parser.add_argument(
        "--chunks",
        type=int,
        nargs=2,
        metavar=("START", "END"),
        default=(None, None),
        help="First and last chunks to plot.",
    )
    parser.add_argument(
        "--ignore",
        type=str,
        nargs="*",
        default=[],
        help="Ignore jobs starting with these arguments.",
    )
    parser.add_argument(
        "--color",
        choices=("job", "chunk", "status"),
        default="chunk",
        help="Whether to color jobs by job, by chunk or by status.",
    )
    parser.add_argument(
        "--starttype",
        choices=("started", "submitted"),
        default="started",
        help="Whether the start of the bars is at AS 'started' or 'submitted' status.",
    )
    parser.add_argument(
        "--cmap", type=str, default="tab20", help="Matplotlib colormap to use."
    )
    parser.add_argument(
        "--interval",
        type=int,
        default=-1,
        help="Interval for time ticks in x axis. -1 sets it to 'automatic'. Default: -1",
    )
    parser.add_argument(
        "--pivot",
        action="store_true",
        help="All bars are classified by chunk in the y axis.",
    )
    parser.add_argument(
        "--no-pivot",
        action="store_false",
        dest="autopivot",
        help="Prevent bars from being classified by chunk in the y axis, even if splits overlap.",
    )
    parser.add_argument(
        "--by-split",
        action="store_true",
        help="Gantt Chart entries are separated by split.",
    )
    parser.add_argument(
        "--durations",
        action="store_true",
        help="Add the summed durations of all the jobs of each entry in the Gantt chart.",
    )
    parser.set_defaults(autopivot=True)
    return parser


def main():
    args = _get_parser().parse_args()

    expid = args.expid
    chunks0, chunks1 = args.chunks
    ignore = args.ignore
    color = args.color
    starttype = args.starttype
    cmap = plt.get_cmap(args.cmap)
    interval = args.interval
    pivot = args.pivot
    autopivot = args.autopivot
    by_split = args.by_split
    durations = args.durations

    print("Reading logs...")
    experiment_path = LOG_DIR / expid / "ASFILES"
    aslogs = get_aslogs_df(experiment_path, ignore_prefix=IGNORE_PREFIX)

    print("Extracting jobs...")
    jobs = jobs_df_from_aslogs_df(aslogs, experiment_path)

    if chunks0 is not None and chunks1 is not None:
        jobs = jobs[
            (jobs["chunk"] >= chunks0)
            & (jobs["chunk"] <= chunks1)
            & (
                jobs["job"].apply(
                    lambda x: not any(x.startswith(element) for element in ignore)
                )
            )
        ]
        title = f"{expid} Simulation Timeline - Chunks [{chunks0}, {chunks1}]"
        filename = f"{expid}_simulation_timeline_{chunks0}_{chunks1}.png"
    else:
        title = f"{expid} Simulation Timeline"
        filename = f"{expid}_simulation_timeline.png"

    print("Generating Gantt Chart plot")
    fig = plot_job_gantt_chart(
        jobs,
        title=title,
        color_key=color,
        starttype=starttype,
        cmap=cmap,
        interval=interval,
        pivot=(
            "split"
            if by_split
            else "chunk" if pivot else "on_overlap" if autopivot else False
        ),
        add_durations=durations,
    )
    Path("figures").mkdir(exist_ok=True)
    fig.savefig(f"figures/{filename}", dpi=300)
    print(f"Saved figures/{filename}")


if __name__ == "__main__":
    main()
